@@ -1,4 +1,4 @@
-/* GnoRPM - A GNOME front end for the Redhat Package Manager (RPM)
+/* GnoRPM - A GNOME front end for the Red Hat Package Manager (RPM)
  * Copyright (C) 1998-1999  James Henstridge
  *
  * This program is free software; you can redistribute it and/or modify
@@ -25,9 +25,12 @@
 
 #include "dbhandle.h"
 #include "install.h"
+#include "checksig.h"
+#include "verify.h"
 #include "misc.h"
 
 #include "rpmquerydlg.h"
+#include "rpmprops.h"
 
 #ifdef WITH_RPMFIND
 #include "find/libfind.h"
@@ -65,7 +68,7 @@ static gchar *root = "";
 static GList *args = NULL;
 static enum { MODE_UNSET, MODE_QUERY, MODE_INSTALL, MODE_UPGRADE}
   majorMode = MODE_UNSET;
-static isFiles = FALSE;
+static int isFiles = FALSE;
 static GdkWindow *main_window = NULL;
 
 static void parseAnArg(poptContext con, enum poptCallbackReason reason,
@@ -109,14 +112,111 @@ static void update_func(void) {
     gtk_main_iteration();
 }
 
+/* these functions handle the buttons in the "gnorpm -q" window */
+static void query_verify_func(RpmQuery *info, gpointer data) {
+  verify_one(info->hdl, info->index, 0);
+}
+static void query_uninstall_func(RpmQuery *info, gpointer data) {
+  int failed;
+  char buf[512];
+#ifdef HAVE_RPM_3_0
+  guint interfaceFlags = 0, transFlags = 0, probFilter = 0;
+  rpm_props_box_get_flags(&interfaceFlags, &transFlags, &probFilter);
+#else    
+  guint interfaceFlags = 0, installFlags = 0, uninstallFlags = 0;
+  rpm_props_box_get_flags(&interfaceFlags, &installFlags, &uninstallFlags);
+#endif
+
+  db_handle_db_down(info->hdl);
+#ifdef HAVE_RPM_3_0
+  failed = uninstall_one(info->hdl->root, info->index, transFlags,
+                         probFilter, interfaceFlags);
+#else
+  failed = uninstall_one(info->hdl->root, info->index, uninstallFlags,
+                         interfaceFlags);
+#endif
+  db_handle_db_up(info->hdl);
+  if (failed) {
+    g_snprintf(buf, 511, _("%d packages couldn't be uninstalled"), failed);
+    message_box(buf);
+    return;
+  }
+#ifdef HAVE_RPM_3_0
+  if (!(transFlags & RPMTRANS_FLAG_TEST)) {
+#else
+    if (!(uninstallFlags & RPMUNINSTALL_TEST)) {
+#endif
+    GtkWidget *parent = GTK_WIDGET(info)->parent;
+
+    gtk_container_remove(GTK_CONTAINER(parent), GTK_WIDGET(info));
+  }
+}
+static void query_install_func(RpmQuery *info, gpointer data) {
+#ifdef HAVE_RPM_3_0
+  guint interfaceFlags = 0, transFlags = 0, probFilter = 0;
+  rpm_props_box_get_flags(&interfaceFlags, &transFlags, &probFilter);
+#else    
+  guint interfaceFlags = 0, installFlags = 0, uninstallFlags = 0;
+  rpm_props_box_get_flags(&interfaceFlags, &installFlags, &uninstallFlags);
+#endif
+
+  db_handle_db_down(info->hdl);
+#ifdef HAVE_RPM_3_0
+  if (install_one(info->hdl->root, info->fname, NULL, transFlags, probFilter,
+                  interfaceFlags, NULL, NULL)) {
+#else
+  if (install_one(info->hdl->root, info->fname, NULL, installFlags,
+                  interfaceFlags, NULL, NULL)) {
+#endif
+    gchar buf[512];
+    g_snprintf(buf, 511, _("Couldn't install %s"), info->fname);
+    message_box(buf);
+  }
+  db_handle_db_up(info->hdl);
+}
+static void query_upgrade_func(RpmQuery *info, gpointer data) {
+#ifdef HAVE_RPM_3_0
+  guint interfaceFlags = 0, transFlags = 0, probFilter = 0;
+  rpm_props_box_get_flags(&interfaceFlags, &transFlags, &probFilter);
+#else    
+  guint interfaceFlags = 0, installFlags = 0, uninstallFlags = 0;
+  rpm_props_box_get_flags(&interfaceFlags, &installFlags, &uninstallFlags);
+#endif
+
+  db_handle_db_down(info->hdl);
+#ifdef HAVE_RPM_3_0
+  if (install_one(info->hdl->root, info->fname, NULL,
+                  transFlags, probFilter, interfaceFlags | INTER_UPGRADE,
+                  NULL, NULL)) {
+#else
+  if (install_one(info->hdl->root, info->fname, NULL,
+                  installFlags | RPMINSTALL_UPGRADE, interfaceFlags,
+                  NULL, NULL)) {
+#endif      
+    gchar buf[512];
+    g_snprintf(buf, 511, _("Couldn't upgrade %s"), info->fname);
+    message_box(buf);
+  }
+  db_handle_db_up(info->hdl);
+}
+static void query_checksig_func(RpmQuery *info, gpointer data) {
+  check_one_sig(info->fname, 0);
+}
+
+
 int main(int argc, char *argv[]) {
   GtkWidget *app;
   GnomeClient *client;
   DBHandle *dbhdl = NULL;
   GList *pkgs = NULL, *tmp;
   dbiIndexSet matches;
-  gint i, installFlags = 0, res;
+  gint i, res;
   gchar buf[512];
+#ifdef HAVE_RPM_3_0
+  int interfaceFlags = 0;
+#else
+  int installFlags = 0;
+#endif
 
   /* i18n stuff */
   bindtextdomain(PACKAGE, GNOMELOCALEDIR);
@@ -127,9 +227,15 @@ int main(int argc, char *argv[]) {
   client = gnome_master_client();
   gtk_signal_connect(GTK_OBJECT(client), "save_yourself",
 		     GTK_SIGNAL_FUNC(save_state), argv[0]);
+  gtk_signal_connect(GTK_OBJECT(client), "die",
+		     GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
 
+#ifdef HAVE_RPM_3_0 /* RPM_3_0 */
+  rpmReadConfigFiles(NULL, NULL);
+#else
   if (rpmReadConfigFiles(NULL, NULL, NULL, 0))
     return 1;
+#endif
   setupErrorHandler();
 
   if (majorMode == MODE_UNSET || majorMode == MODE_QUERY) {
@@ -142,7 +248,7 @@ int main(int argc, char *argv[]) {
     libfind_set_idle_func(update_func);
 #endif
   }
-  
+
   switch (majorMode) {
   case MODE_UNSET:
     app = create_main("gnorpm", dbhdl);
@@ -155,9 +261,13 @@ int main(int argc, char *argv[]) {
     gtk_main();
     break;
   case MODE_QUERY:
-    if (isFiles)
+    if (isFiles) {
+      if (!args) {
+	message_box(_("You must give a package name to query"));
+	return 1;
+      }
       app = rpm_query_dialog_new_from_files(dbhdl, args);
-    else {
+    } else {
       pkgs = NULL;
       for (tmp = args; tmp != NULL; tmp = tmp->next) {
 	if (rpmdbFindByLabel(dbhdl->db, tmp->data, &matches))
@@ -168,6 +278,10 @@ int main(int argc, char *argv[]) {
 				 GINT_TO_POINTER(matches.recs[i].recOffset));
 	dbiFreeIndexRecord(matches);
       }
+      if (!pkgs) {
+	message_box(_("You must give a package name to query"));
+	return 1;
+      }
       app = rpm_query_dialog_new(dbhdl, pkgs);
     }
     if (main_w != -1 || main_h != -1)
@@ -176,14 +290,34 @@ int main(int argc, char *argv[]) {
       gtk_widget_set_uposition(app, main_x, main_y);
     gtk_signal_connect(GTK_OBJECT(app), "destroy",
 		       GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
+    rpm_query_dialog_set_uninstall_func(RPM_QUERY_DIALOG(app),
+					query_uninstall_func, NULL);
+    rpm_query_dialog_set_verify_func(RPM_QUERY_DIALOG(app),
+				     query_verify_func, NULL);
+    rpm_query_dialog_set_install_func(RPM_QUERY_DIALOG(app),
+				      query_install_func, NULL);
+    rpm_query_dialog_set_upgrade_func(RPM_QUERY_DIALOG(app),
+				      query_upgrade_func, NULL);
+    rpm_query_dialog_set_checksig_func(RPM_QUERY_DIALOG(app),
+				      query_checksig_func, NULL);
+    rpm_query_dialog_set_close_func(RPM_QUERY_DIALOG(app),
+				    (GtkRpmCallback)gtk_main_quit, NULL);
     gtk_widget_show(app);
     main_window = app->window;
     gtk_main();
     break;
   case MODE_UPGRADE:
+#ifdef HAVE_RPM_3_0
+    interfaceFlags = interfaceFlags | INTER_UPGRADE;
+#else
     installFlags = installFlags | RPMINSTALL_UPGRADE;
+#endif
   case MODE_INSTALL:
-    res = do_install(root, pkgs, NULL, installFlags, 0, NULL, NULL);
+#ifdef HAVE_RPM_3_0
+    res = do_install(root, args, NULL, 0, 0, interfaceFlags, NULL, NULL);
+#else
+    res = do_install(root, args, NULL, installFlags, 0, NULL, NULL);
+#endif
     if (res) {
       g_snprintf(buf, 511, _("%s of %d packages failed."),
 		 (majorMode==MODE_UPGRADE)?_("Upgrade"):_("Install"), res);
@@ -247,4 +381,3 @@ static gint save_state(GnomeClient *client, gint phase,
   if (geom) g_free(geom);
   return FALSE;
 }
-
