@@ -1,4 +1,4 @@
-/* GnoRPM - A GNOME front end for the Redhat Package Manager (RPM)
+/* GnoRPM - A GNOME front end for the Red Hat Package Manager (RPM)
  * Copyright (C) 1998-1999  James Henstridge
  *
  * This program is free software; you can redistribute it and/or modify
@@ -40,9 +40,6 @@
 #include "rpmwebfind.h"
 #endif
 
-/* the pixmap ... */
-#include "boom.xpm"
-
 GnomeApp *gnorpm_app = NULL;
 
 enum { TARGET_URI_LIST };
@@ -64,21 +61,31 @@ static void mainwin_drop_cb(GnomeApp *app, GdkDragContext *context,
 static void mainwin_context_menu(RpmPackageList *pl, GdkEventButton *event,
 				 guint index);
 
+#ifdef HAVE_RPM_3_0
+static guint interfaceFlags = 0, transFlags = 0, probFilter = 0;
+#else
 static guint interfaceFlags = 0, installFlags = 0, uninstallFlags = 0;
+#endif
 
 #ifdef WITH_RPMFIND
 static void gnorpm_download_start(char *url) {
   char *msg = g_strconcat("Downloading ", url, " ...", NULL);
   GnomeAppProgressKey key = gnome_app_progress_manual(gnorpm_app, msg,
 						      NULL, NULL);
+  GSList *list;
 
-  gtk_object_set_data(GTK_OBJECT(gnorpm_app), "download_key", key);
+  list = gtk_object_get_data(GTK_OBJECT(gnorpm_app), "download_key");
+  list = g_slist_prepend(list, key);
+  gtk_object_set_data(GTK_OBJECT(gnorpm_app), "download_key", list);
   g_free(msg);
 }
 
 static void gnorpm_download_progress(unsigned long done, unsigned long total) {
-  GnomeAppProgressKey key = gtk_object_get_data(GTK_OBJECT(gnorpm_app),
-						"download_key");
+  GSList *list = gtk_object_get_data(GTK_OBJECT(gnorpm_app), "download_key");
+  GnomeAppProgressKey key;
+
+  g_return_if_fail(list != NULL);
+  key = (GnomeAppProgressKey) list->data;
 
   if (total == 0) total = 1;
   gnome_app_set_progress(key, ((gdouble)done)/total);
@@ -88,12 +95,16 @@ static void gnorpm_download_progress(unsigned long done, unsigned long total) {
 }
 
 static void gnorpm_download_done(void) {
-  GnomeAppProgressKey key = gtk_object_get_data(GTK_OBJECT(gnorpm_app),
-						"download_key");
+  GSList *list = gtk_object_get_data(GTK_OBJECT(gnorpm_app), "download_key");
+  GnomeAppProgressKey key;
+
+  g_return_if_fail(list != NULL);
+  key = (GnomeAppProgressKey) list->data;
+  list = g_slist_remove(list, key);
 
   gnome_app_progress_done(key);
   gnome_appbar_refresh(GNOME_APPBAR(gnorpm_app->statusbar));
-  gtk_object_set_data(GTK_OBJECT(gnorpm_app), "download_key", NULL);
+  gtk_object_set_data(GTK_OBJECT(gnorpm_app), "download_key", list);
 }
 #endif
 
@@ -102,7 +113,6 @@ GtkWidget *create_main(char *app_id, DBHandle *hdl) {
 
   app = gnome_app_new(app_id, _("Gnome RPM"));
   gnorpm_app = GNOME_APP(app);
-  gtk_widget_set_usize(app, 650, 350);
   gtk_signal_connect(GTK_OBJECT(app), "destroy",
 		     GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
 
@@ -129,10 +139,14 @@ GtkWidget *create_main(char *app_id, DBHandle *hdl) {
 		    gnorpm_download_progress,
 		    gnorpm_download_done);
 #endif
-  
-  /* get the saved properties */
-  rpm_props_box_get_flags(&interfaceFlags, &installFlags, &uninstallFlags);
 
+  /* get the saved properties */
+#ifdef HAVE_RPM_3_0
+  rpm_props_box_get_flags(&interfaceFlags, &transFlags, &probFilter);
+#else    
+  rpm_props_box_get_flags(&interfaceFlags, &installFlags, &uninstallFlags);
+#endif
+  
   if (gnome_config_get_bool("/gnorpm/Layout/asIcons=true"))
     rpm_package_list_set_mode(RPM_PACKAGE_LIST(pl), RPM_PACKAGE_ILIST);
 
@@ -146,6 +160,9 @@ GtkWidget *create_main(char *app_id, DBHandle *hdl) {
   gtk_signal_connect(GTK_OBJECT(app), "drag_data_received",
 		     GTK_SIGNAL_FUNC(mainwin_drop_cb), pl);
 
+  gtk_widget_set_usize(app, 650, 350);
+  gtk_window_set_policy(GTK_WINDOW(app), TRUE, TRUE, FALSE);
+
   return app;
 }
 
@@ -202,7 +219,7 @@ static void create_menus(GtkWidget *app, GtkWidget *pl) {
     GNOMEUIINFO_ITEM_STOCK_DATA(N_("_Verify"),
 			N_("Verify the selected packages"),
 			rpm_verify_pkgs, pl, RPM_STOCK_MENU_VERIFY),
-    GNOMEUIINFO_ITEM_STOCK_DATA(N_("_Create desktop entry..."), 
+    GNOMEUIINFO_ITEM_STOCK_DATA(N_("_Create desktop entry..."),
 	N_("Create desktop entries (for panel) for the selected packages"),
 				rpm_dentry_pkgs,pl, GNOME_STOCK_MENU_BLANK),
     GNOMEUIINFO_SEPARATOR,
@@ -227,7 +244,7 @@ static void create_menus(GtkWidget *app, GtkWidget *pl) {
 				N_("Upgrade some current packages"),
 				rpm_upgrade_pkgs, pl, RPM_STOCK_MENU_UPGRADE),
     GNOMEUIINFO_SEPARATOR,
-    GNOMEUIINFO_ITEM_STOCK_DATA(N_("_Preferences..."), 
+    GNOMEUIINFO_ITEM_STOCK_DATA(N_("_Preferences..."),
 				N_("Alter GnoRPM's preferences"),
 				rpm_preferences_cb, pl, GNOME_STOCK_MENU_PREF),
     GNOMEUIINFO_END
@@ -287,7 +304,12 @@ static void rpm_clear_selection(GtkWidget *mi, RpmPackageList *pl) {
 
 static void rpm_uninstall_pkgs(GtkWidget *mi, RpmPackageList *pl) {
   db_handle_db_down(pl->hdl);
+#ifdef HAVE_RPM_3_0
+  do_uninstall(pl->hdl->root, pl->selection, transFlags, probFilter,
+	       interfaceFlags);
+#else
   do_uninstall(pl->hdl->root, pl->selection, uninstallFlags, interfaceFlags);
+#endif
   db_handle_db_up(pl->hdl);
   /* update the package list display */
   rpm_package_list_clear_selection(pl);
@@ -301,14 +323,26 @@ static void rpm_verify_pkgs(GtkWidget *mi, RpmPackageList *pl) {
 static void verify_func(RpmQuery *info, gpointer data) {
   verify_one(info->hdl, info->index, 0);
 }
+
+static void close_func(RpmQuery *info, gpointer data) {
+  GtkWidget *win = (GtkWidget *) data;
+    
+  gtk_widget_destroy(win);
+}
+
 static void uninstall_func(RpmQuery *info, gpointer data) {
   int failed;
   char buf[512];
   RpmPackageList *pl = RPM_PACKAGE_LIST(data);
 
   db_handle_db_down(info->hdl);
+#ifdef HAVE_RPM_3_0
+  failed = uninstall_one(info->hdl->root, info->index, transFlags,
+			 probFilter, interfaceFlags);
+#else
   failed = uninstall_one(info->hdl->root, info->index, uninstallFlags,
 			 interfaceFlags);
+#endif
   db_handle_db_up(info->hdl);
   if (failed) {
     g_snprintf(buf, 511, _("%d packages couldn't be uninstalled"), failed);
@@ -320,7 +354,11 @@ static void uninstall_func(RpmQuery *info, gpointer data) {
   rpm_package_list_unselect(pl, info->index);
 
   /* remove the package from the query window */
-  if (!(uninstallFlags & RPMUNINSTALL_TEST)) {
+#ifdef HAVE_RPM_3_0
+  if (!(transFlags & RPMTRANS_FLAG_TEST)) {
+#else
+    if (!(uninstallFlags & RPMUNINSTALL_TEST)) {
+#endif
     GtkWidget *parent = GTK_WIDGET(info)->parent;
 
     gtk_container_remove(GTK_CONTAINER(parent), GTK_WIDGET(info));
@@ -335,6 +373,7 @@ static void rpm_query_pkgs(GtkWidget *mi, RpmPackageList *pl) {
   rpm_query_dialog_set_verify_func(RPM_QUERY_DIALOG(win), verify_func, NULL);
   rpm_query_dialog_set_uninstall_func(RPM_QUERY_DIALOG(win), uninstall_func,
 				      pl);
+  rpm_query_dialog_set_close_func(RPM_QUERY_DIALOG(win), close_func, win);
   gtk_widget_show(win);
 }
 
@@ -346,7 +385,7 @@ static void rpm_about(GtkWidget *mi) {
                           "Copyright (C) 1998-1999, James Henstridge",
                           authors,
                           _("May be distributed under the terms of the GPL2\n"
-                          "This program uses rpmlib, written by Redhat"),
+                          "This program uses rpmlib, written by Red Hat"),
                           NULL);
   gtk_widget_show(about);
 }
@@ -360,6 +399,7 @@ static void rpm_find_query_cb(RpmFindDialog *dlg, GList *indices,
   rpm_query_dialog_set_verify_func(RPM_QUERY_DIALOG(win), verify_func, NULL);
   rpm_query_dialog_set_uninstall_func(RPM_QUERY_DIALOG(win), uninstall_func,
 				      pl);
+  rpm_query_dialog_set_close_func(RPM_QUERY_DIALOG(win), close_func, win);
   gtk_widget_show(win);
 }
 
@@ -371,7 +411,12 @@ static void rpm_find_uninstall_cb(RpmFindDialog *dlg, GList *indices,
 
   if (indices == NULL) return;
   db_handle_db_down(pl->hdl);
+#ifdef HAVE_RPM_3_0
+  failed = do_uninstall(pl->hdl->root, indices, transFlags, probFilter,
+			interfaceFlags);
+#else
   failed = do_uninstall(pl->hdl->root, indices, uninstallFlags,interfaceFlags);
+#endif
   db_handle_db_up(pl->hdl);
   if (failed) {
     g_snprintf(buf, 511, _("%d packages couldn't be uninstalled"), failed);
@@ -410,8 +455,13 @@ static void gnorpm_update_tree(gchar *pkgName, gchar *groupName, void *data) {
 
 static void install_func(RpmQuery *info, RpmPackageList *pl) {
   db_handle_db_down(info->hdl);
+#ifdef HAVE_RPM_3_0
+  if (install_one(info->hdl->root, info->fname, NULL, transFlags, probFilter,
+		  interfaceFlags, gnorpm_update_tree, pl)) {
+#else
   if (install_one(info->hdl->root, info->fname, NULL, installFlags,
 		  interfaceFlags, gnorpm_update_tree, pl)) {
+#endif
     gchar buf[512];
     g_snprintf(buf, 511, _("Couldn't install %s"), info->fname);
     message_box(buf);
@@ -421,9 +471,15 @@ static void install_func(RpmQuery *info, RpmPackageList *pl) {
 }
 static void upgrade_func(RpmQuery *info, RpmPackageList *pl) {
   db_handle_db_down(info->hdl);
+#ifdef HAVE_RPM_3_0
+  if (install_one(info->hdl->root, info->fname, NULL,
+		  transFlags, probFilter, interfaceFlags | INTER_UPGRADE,
+		  gnorpm_update_tree, pl)) {
+#else
   if (install_one(info->hdl->root, info->fname, NULL,
 		  installFlags | RPMINSTALL_UPGRADE, interfaceFlags,
 		  gnorpm_update_tree, pl)) {
+#endif      
     gchar buf[512];
     g_snprintf(buf, 511, _("Couldn't upgrade %s"), info->fname);
     message_box(buf);
@@ -447,6 +503,8 @@ static void rpm_instpkgs_query(RpmInstallDialog *dlg, GList *files,
 				    (GtkRpmCallback)upgrade_func, pl);
   rpm_query_dialog_set_checksig_func(RPM_QUERY_DIALOG(query),
 				      (GtkRpmCallback)checksig_func, NULL);
+  rpm_query_dialog_set_close_func(RPM_QUERY_DIALOG(query),
+				      (GtkRpmCallback)close_func, query);
   gtk_widget_show(query);
 }
 
@@ -459,8 +517,13 @@ static void rpm_install_pkgs_cb(RpmInstallDialog *dlg, GList *files,
     return;
   }
   db_handle_db_down(dlg->hdl);
+#ifdef HAVE_RPM_3_0
+  res = do_install(dlg->hdl->root, files, NULL, transFlags, interfaceFlags,
+		   probFilter, gnorpm_update_tree, pl);
+#else
   res = do_install(dlg->hdl->root, files, NULL, installFlags, interfaceFlags,
 		   gnorpm_update_tree, pl);
+#endif
   if (res) {
     gchar buf[512];
     g_snprintf(buf, 511, _("Install of %d packages failed."), res);
@@ -527,9 +590,16 @@ static void rpm_upgrade_pkgs_cb(RpmInstallDialog *dlg, GList *files,
     return;
   }
   db_handle_db_down(dlg->hdl);
+#ifdef HAVE_RPM_3_0
+  res = do_install(dlg->hdl->root, files, NULL,
+		   transFlags, interfaceFlags | INTER_UPGRADE,
+		   probFilter, gnorpm_update_tree, pl);
+#else
   res = do_install(dlg->hdl->root, files, NULL,
 		   installFlags | RPMINSTALL_UPGRADE, interfaceFlags,
 		   gnorpm_update_tree, pl);
+#endif
+
   if (res) {
     gchar buf[512];
     g_snprintf(buf, 511, _("Upgrade of %d packages failed."), res);
@@ -556,6 +626,9 @@ static void rpm_show_web_find(GtkWidget *mi, RpmPackageList *pl) {
   GtkWidget *win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
   GtkWidget *webfind = rpm_web_find_new(pl);
 
+  gtk_signal_connect_object(GTK_OBJECT(RPM_WEB_FIND(webfind)->info), "close",
+			    GTK_SIGNAL_FUNC(gtk_widget_destroy),
+			    GTK_OBJECT(win));
   gtk_window_set_title(GTK_WINDOW(win), _("Rpmfind"));
   rpm_web_find_set_callbacks(RPM_WEB_FIND(webfind),
 			     GTK_SIGNAL_FUNC(rpm_install_pkgs_cb),
@@ -580,9 +653,15 @@ static void rpm_preferences_apply_cb(RpmPropsBox *box, gint page_num,
 				     RpmPackageList *pl) {
   switch (page_num) {
   case RPM_PROPS_FLAGS:
+#ifdef HAVE_RPM_3_0
+    interfaceFlags = box->interfaceFlags;
+    transFlags     = box->transFlags;
+    probFilter     = box->probFilter;
+#else
     interfaceFlags = box->interfaceFlags;
     installFlags   = box->installFlags;
     uninstallFlags = box->uninstallFlags;
+#endif
     break;
   case RPM_PROPS_INTERFACE:
     if (GTK_TOGGLE_BUTTON(box->asIcons)->active)
@@ -627,16 +706,33 @@ static void context_query(GtkWidget *mi, gpointer index_p) {
   rpm_query_dialog_set_verify_func(RPM_QUERY_DIALOG(win), verify_func, NULL);
   rpm_query_dialog_set_uninstall_func(RPM_QUERY_DIALOG(win), uninstall_func,
 				      pl);
+  rpm_query_dialog_set_close_func(RPM_QUERY_DIALOG(win), close_func, win);  
   gtk_widget_show(win);
 }
 
 static void context_uninstall(GtkWidget *mi, gpointer index_p) {
+  gint failed;
+  char buf[512];
   guint index = GPOINTER_TO_UINT(index_p);
+
   RpmPackageList *pl = gtk_object_get_data(GTK_OBJECT(mi), "package-list");
 
   db_handle_db_down(pl->hdl);
-  uninstall_one(pl->hdl->root, index, uninstallFlags, interfaceFlags);
+#ifdef HAVE_RPM_3_0
+  failed = uninstall_one(pl->hdl->root, index, transFlags, probFilter,
+			 interfaceFlags);
+#else
+  failed = uninstall_one(pl->hdl->root, index, uninstallFlags, interfaceFlags);
+#endif
   db_handle_db_up(pl->hdl);
+  if (failed) {
+    char buf[512];
+    g_snprintf(buf, 512, _("%d packages couldn't be uninstalled"), failed);
+    message_box(buf);
+    return;
+  }
+  rpm_package_list_unselect(pl, index);
+  rpm_package_list_update_pane(pl);
 }
 
 static void context_verify(GtkWidget *mi, gpointer index_p) {
@@ -674,4 +770,3 @@ static void mainwin_context_menu(RpmPackageList *pl, GdkEventButton *event,
   }
   gnome_popup_menu_do_popup(menu, NULL, NULL, event, GUINT_TO_POINTER(index));
 }
-
