@@ -1,4 +1,4 @@
-/* GnoRPM - A GNOME front end for the Redhat Package Manager (RPM)
+/* GnoRPM - A GNOME front end for the Red Hat Package Manager (RPM)
  * Copyright (C) 1998-1999  James Henstridge
  *
  * This program is free software; you can redistribute it and/or modify
@@ -144,9 +144,14 @@ static void rpm_package_list_init(RpmPackageList *self) {
 			    GTK_OBJECT(self));
   gtk_clist_column_titles_passive(GTK_CLIST(self->clist));
   gtk_clist_set_selection_mode(GTK_CLIST(self->clist), GTK_SELECTION_MULTIPLE);
-  gtk_clist_set_column_width(GTK_CLIST(self->clist), 0, 120);
-  gtk_clist_set_column_width(GTK_CLIST(self->clist), 1, 50);
-  gtk_clist_set_column_width(GTK_CLIST(self->clist), 2, 40);
+  gtk_clist_set_column_min_width(GTK_CLIST(self->clist), 0, 120);
+  gtk_clist_set_column_auto_resize(GTK_CLIST(self->clist), 0, TRUE);
+  gtk_clist_set_column_min_width(GTK_CLIST(self->clist), 1, 50);
+  gtk_clist_set_column_auto_resize(GTK_CLIST(self->clist), 1, TRUE);
+  gtk_clist_set_column_min_width(GTK_CLIST(self->clist), 2, 40);
+  gtk_clist_set_column_auto_resize(GTK_CLIST(self->clist), 2, TRUE);
+  gtk_clist_set_column_auto_resize(GTK_CLIST(self->clist), 3, TRUE);
+  gtk_clist_columns_autosize(GTK_CLIST(self->clist));
   gtk_container_add(GTK_CONTAINER(self->clist_cont), self->clist);
   gtk_widget_show(self->clist);
 
