@@ -1,4 +1,4 @@
-/* GnoRPM - A GNOME front end for the Redhat Package Manager (RPM)
+/* GnoRPM - A GNOME front end for the Red Hat Package Manager (RPM)
  * Copyright (C) 1998-1999  James Henstridge
  *
  * This program is free software; you can redistribute it and/or modify
@@ -63,41 +63,108 @@ static void rpm_props_box_class_init(RpmPropsBoxClass *klass) {
   GNOME_PROPERTY_BOX_CLASS(klass)->apply = rpm_props_box_apply;
 }
 
+#ifdef HAVE_RPM_3_0
 typedef struct {
-  gint32 interfaceFlags, installFlags, uninstallFlags;
-  const gchar *tag, *string;
+  gint32 interfaceFlags, transFlags, probFilter;
+  const gchar *tag, *string, *help_string;
 } RpmFlagInfo;
 const static RpmFlagInfo prop_flags[] = {
-  /* install options */
-  {INTER_NODEPS,  0, 0, "NoDependencies", N_("No dependency checks")},
-  {INTER_NOORDER, 0, 0, "NoReorder", N_("No reordering")},
+    /* install options */
+  {INTER_NODEPS,  0, 0, "NoDependencies", N_("No dependency checks"),
+   N_("Do not check for dependencies on other rpms. Equivalent to --nodeps")},
+  {INTER_NOORDER, 0, 0, "NoReorder", N_("No reordering"),
+   N_("Do not reorder package installation to satisfy dependencies. Equivalent to --noorder")},
+  {0, RPMTRANS_FLAG_NOSCRIPTS, 0, "NoScripts",
+   N_("Don't run scripts"),
+   N_("Do not run the pre and post install scripts. Equivalent to --noscripts.")}, 
+  /* upgrade options */
+  {0, 0, RPMPROB_FILTER_REPLACEPKG, "ReplacePkgs",
+   N_("Allow replacement of packages"), 
+   N_("Replase packages with a new copy of itself. Equivalent to --replacepkgs.")},
+  {0, 0, RPMPROB_FILTER_REPLACEOLDFILES, "ReplaceFiles",
+   N_("Allow replacement of files"),
+   N_("Replace files owned by another package. Equivalent to --replacefiles.")},
+  {0, 0, RPMPROB_FILTER_OLDPACKAGE, "UpgradeToOld",
+   N_("Allow upgrade to old version"),
+   N_("Upgrade packages to an older package if need be. Equivalent to --oldpackage")},
+  {0, RPMTRANS_FLAG_KEEPOBSOLETE, 0, "KeepObsolete",
+   N_("Keep packages made obsolete"),
+   N_("If a package has been obsoleted, keep it anyway.")},
+  /* file options */
+  {0, RPMTRANS_FLAG_NODOCS, 0, "NoDocumentation",
+   N_("Don't install documentation"),
+   N_("Do not install any of the doc files. Equivalent to --excludedocs.")},
+  {0, RPMTRANS_FLAG_ALLFILES, 0, "AllFiles",
+   N_("Install all files"),
+   N_("Install all files")},
+  /* database options */
+  {0, RPMTRANS_FLAG_JUSTDB, RPMTRANS_FLAG_JUSTDB, "JustDB",
+   N_("Just update database"),
+   N_("Do not change any files, just update the database as though you did. Equivalent to --justdb") },
+  {0, RPMTRANS_FLAG_TEST, RPMTRANS_FLAG_TEST, "JustTest",
+   N_("Just test"),
+   N_("Do not actually change anything, just test.  Equivalent to --test") },
+  /* architecture checks */
+  {0, 0, RPMPROB_FILTER_IGNOREARCH, "NoArchCheck",
+   N_("Don't check package architecture"), 
+   N_("Do not check the system and rpm architectures. Equivalent to --ignorearch")},
+  {0, 0, RPMPROB_FILTER_IGNOREOS, "NoOSCheck",
+   N_("Don't check package OS"),
+   N_("Do not check the package OS field. Equivalent to --ignoreos")},
+};
+
+#else /* !HAVE_RPM_3_0 */
+
+typedef struct {
+  gint32 interfaceFlags, installFlags, uninstallFlags;
+  const gchar *tag, *string, *help_string;
+} RpmFlagInfo;
+const static RpmFlagInfo prop_flags[] = {
+    /* install options */
+  {INTER_NODEPS,  0, 0, "NoDependencies", N_("No dependency checks"),
+   N_("Do not check for dependencies on other rpms. Equivalent to --nodeps")},
+  {INTER_NOORDER, 0, 0, "NoReorder", N_("No reordering"),
+   N_("Do not reorder package installation to satisfy dependencies. Equivalent to --noorder")},
   {0, RPMINSTALL_NOSCRIPTS, RPMUNINSTALL_NOSCRIPTS, "NoScripts",
-   N_("Don't run scripts")}, 
+   N_("Don't run scripts"),
+   N_("Do not run the pre and post install scripts. Equivalent to --noscripts.")}, 
   /* upgrade options */
   {0, RPMINSTALL_REPLACEPKG, 0, "ReplacePkgs",
-   N_("Allow replacement of packages")},
+   N_("Allow replacement of packages"), 
+   N_("Replase packages with a new copy of itself. Equivalent to --replacepkgs.")},
   {0, RPMINSTALL_REPLACEFILES, 0, "ReplaceFiles",
-   N_("Allow replacement of files")},
+   N_("Allow replacement of files"),
+   N_("Replace files owned by another package. Equivalent to --replacefiles.")},
   {0, RPMINSTALL_UPGRADETOOLD, 0, "UpgradeToOld",
-   N_("Allow upgrade to old version")},
+   N_("Allow upgrade to old version"),
+   N_("Upgrade packages to an older package if need be. Equivalent to --oldpackage")},
   {0, RPMINSTALL_KEEPOBSOLETE, 0, "KeepObsolete",
-   N_("Keep packages made obsolete")},
+   N_("Keep packages made obsolete"),
+   N_("If a package has been obsoleted, keep it anyway.")},
   /* file options */
   {0, RPMINSTALL_NODOCS, 0, "NoDocumentation",
-   N_("Don't install documentation")},
+   N_("Don't install documentation"),
+   N_("Do not install any of the doc files. Equivalent to --excludedocs.")},
   {0, RPMINSTALL_ALLFILES, 0, "AllFiles",
+   N_("Install all files"),
    N_("Install all files")},
   /* database options */
   {0, RPMINSTALL_JUSTDB, RPMUNINSTALL_JUSTDB, "JustDB",
-   N_("Just update database")},
+   N_("Just update database"),
+   N_("Do not change any files, just update the database as though you did. Equivalent to --justdb") },
   {0, RPMINSTALL_TEST, RPMUNINSTALL_TEST, "JustTest",
-   N_("Just test")},
+   N_("Just test"),
+   N_("Do not actually change anything, just test.  Equivalent to --test") },
   /* architecture checks */
   {0, RPMINSTALL_NOARCH, 0, "NoArchCheck",
-   N_("Don't check package architecture")},
+   N_("Don't check package architecture"), 
+   N_("Do not check the system and rpm architectures. Equivalent to --ignorearch")},
   {0, RPMINSTALL_NOOS, 0, "NoOSCheck",
-   N_("Don't check package OS")},
+   N_("Don't check package OS"),
+   N_("Do not check the package OS field. Equivalent to --ignoreos")},
 };
+#endif /* !HAVE_RPM_3_0 */
+
 const int num_prop_flags = sizeof(prop_flags) / sizeof(RpmFlagInfo);
 
 static GtkWidget *rpm_props_box_fill_frame(RpmPropsBox *self, gchar *name,
@@ -112,6 +179,7 @@ static void rpm_props_upgrade_del(RpmPropsBox *self);
 
 static void rpm_props_box_init(RpmPropsBox *self) {
   GtkWidget *frame, *hbox, *vbox, *box, *table, *label, *wid;
+  GtkWidget *tooltip;
   gchar *text;
   gchar *distTitles[4] = { N_("Name"), N_("ID"), N_("Rating"), N_("mirror") };
   GList *tmp;
@@ -119,7 +187,11 @@ static void rpm_props_box_init(RpmPropsBox *self) {
   set_icon(GTK_WIDGET(self));
   gtk_window_set_title(GTK_WINDOW(self), _("Preferences"));
 
+#ifdef HAVE_RPM_3_0
+  self->interfaceFlags = self->transFlags = self->probFilter = 0;
+#else
   self->interfaceFlags = self->installFlags = self->uninstallFlags = 0;
+#endif
   self->flags = g_new(GtkWidget *, num_prop_flags);
   /* create the behviour page */
   hbox = gtk_hbox_new(FALSE, GNOME_PAD);
@@ -197,11 +269,9 @@ static void rpm_props_box_init(RpmPropsBox *self) {
   gtk_container_add(GTK_CONTAINER(frame), box);
   gtk_widget_show(box);
 
-  self->rpmPath = gtk_entry_new();
-  text =gnome_config_get_string("/gnorpm/Paths/rpmDir=/mnt/cdrom/RedHat/RPMS");
-  gtk_entry_set_text(GTK_ENTRY(self->rpmPath), text);
-  g_free(text);
-  gtk_signal_connect_object(GTK_OBJECT(self->rpmPath), "changed",
+  self->rpmPath = gnome_file_entry_new("RpmPath","RpmPath");
+  gnome_file_entry_set_directory(GNOME_FILE_ENTRY(self->rpmPath), TRUE);
+  gtk_signal_connect_object(GTK_OBJECT(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(self->rpmPath))), "changed",
 			    GTK_SIGNAL_FUNC(gnome_property_box_changed),
 			    GTK_OBJECT(self));
   gtk_box_pack_start(GTK_BOX(box), self->rpmPath, TRUE, TRUE, 0);
@@ -284,11 +354,14 @@ static void rpm_props_box_init(RpmPropsBox *self) {
   gtk_table_attach(GTK_TABLE(table), label, 0,1, 3,4,
 		   GTK_FILL, GTK_FILL|GTK_EXPAND, 0, 0);
   gtk_widget_show(label);
-  self->downloadDir = gtk_entry_new();
+
+  self->downloadDir = gnome_file_entry_new("Foo", "Download Dir");
+  gnome_file_entry_set_directory(GNOME_FILE_ENTRY(self->downloadDir), TRUE);
   text = gnome_config_get_string("/gnorpm/rpmfind/rpm-dir=");
-  gtk_entry_set_text(GTK_ENTRY(self->downloadDir), text);
+  gnome_file_entry_set_default_path(GNOME_FILE_ENTRY(self->downloadDir),text);
+  gtk_entry_set_text(GTK_ENTRY(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(self->downloadDir))), text);
   g_free(text);
-  gtk_signal_connect_object(GTK_OBJECT(self->downloadDir), "changed",
+  gtk_signal_connect_object(GTK_OBJECT(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(self->downloadDir))), "changed",
 			    GTK_SIGNAL_FUNC(gnome_property_box_changed),
 			    GTK_OBJECT(self));
   gtk_table_attach(GTK_TABLE(table), self->downloadDir, 1,4, 3,4,
@@ -387,7 +460,7 @@ static void rpm_props_box_init(RpmPropsBox *self) {
   gtk_widget_show(vbox);
   gtk_widget_show(label);
   gnome_property_box_append_page(GNOME_PROPERTY_BOX(self), vbox, label);
-  
+
   /* distributions page */
   vbox = gtk_vbox_new(FALSE, GNOME_PAD);
   gtk_container_set_border_width(GTK_CONTAINER(vbox), GNOME_PAD);
@@ -450,7 +523,7 @@ static void rpm_props_box_init(RpmPropsBox *self) {
   gtk_table_attach(GTK_TABLE(table), self->distName, 1,3, 1,2,
 		   GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 0, 0);
   gtk_widget_show(self->distName);
-  
+
   label = gtk_label_new(_("Origin:"));
   gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
   gtk_table_attach(GTK_TABLE(table), label, 0,1, 2,3,
@@ -586,6 +659,17 @@ GtkWidget *rpm_props_box_new(void) {
 
 static void rpm_props_box_flag_toggled(GtkToggleButton *cb, RpmPropsBox *self){
   RpmFlagInfo *info = gtk_object_get_user_data(GTK_OBJECT(cb));
+#ifdef HAVE_RPM_3_0
+  if (cb->active) {
+    self->interfaceFlags |= info->interfaceFlags;
+    self->transFlags     |= info->transFlags;
+    self->probFilter     |= info->probFilter;
+  } else {
+    self->interfaceFlags &= ~info->interfaceFlags;
+    self->transFlags     &= ~info->transFlags;
+    self->probFilter     &= ~info->probFilter;
+  }
+#else
   if (cb->active) {
     self->interfaceFlags |= info->interfaceFlags;
     self->installFlags   |= info->installFlags;
@@ -595,6 +679,7 @@ static void rpm_props_box_flag_toggled(GtkToggleButton *cb, RpmPropsBox *self){
     self->installFlags   &= ~info->installFlags;
     self->uninstallFlags &= ~info->uninstallFlags;
   }
+#endif
   gnome_property_box_changed(GNOME_PROPERTY_BOX(self));
 }
 
@@ -603,6 +688,7 @@ static GtkWidget *rpm_props_box_fill_frame(RpmPropsBox *self, gchar *name,
   GtkWidget *frame, *box;
   gint i, state;
   gchar buf[1024];
+  GtkTooltips *tooltip;
 
   frame = gtk_frame_new(name);
   box = gtk_vbox_new(FALSE, GNOME_PAD_SMALL);
@@ -612,12 +698,21 @@ static GtkWidget *rpm_props_box_fill_frame(RpmPropsBox *self, gchar *name,
   for (i = start; i < end+1; i++) {
     self->flags[i] = gtk_check_button_new_with_label(_(prop_flags[i].string));
     /* get old value */
+    tooltip = gtk_tooltips_new();
+    gtk_tooltips_set_tip(tooltip, self->flags[i],
+			 (_(prop_flags[i].help_string)) , NULL);
     g_snprintf(buf, 1023, "/gnorpm/Flags/%s=false", prop_flags[i].tag);
     state = gnome_config_get_bool(buf);
     if (state) {
+#ifdef HAVE_RPM_3_0
+      self->interfaceFlags |= prop_flags[i].interfaceFlags;
+      self->transFlags     |= prop_flags[i].transFlags;
+      self->probFilter     |= prop_flags[i].probFilter;
+#else
       self->interfaceFlags |= prop_flags[i].interfaceFlags;
       self->installFlags |= prop_flags[i].installFlags;
       self->uninstallFlags |= prop_flags[i].uninstallFlags;
+#endif
     }
     gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(self->flags[i]), state);
     gtk_object_set_user_data(GTK_OBJECT(self->flags[i]),
@@ -721,7 +816,7 @@ static void rpm_props_box_apply(GnomePropertyBox *object, gint page_num) {
     gnome_config_set_bool("/gnorpm/Layout/asIcons",
 			  GTK_TOGGLE_BUTTON(self->asIcons)->active);
     gnome_config_set_string("/gnorpm/Paths/rpmDir",
-			    gtk_entry_get_text(GTK_ENTRY(self->rpmPath)));
+			    gtk_entry_get_text(GTK_ENTRY(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(self->rpmPath)))));
     break;
 #ifdef WITH_RPMFIND
   case RPM_PROPS_RPMFIND:
@@ -732,7 +827,7 @@ static void rpm_props_box_apply(GnomePropertyBox *object, gint page_num) {
     gnome_config_set_string("/gnorpm/rpmfind/ftp-proxy",
 			    gtk_entry_get_text(GTK_ENTRY(self->ftpProxy)));
     gnome_config_set_string("/gnorpm/rpmfind/rpm-dir",
-			    gtk_entry_get_text(GTK_ENTRY(self->downloadDir)));
+			    gnome_file_entry_get_full_path(GNOME_FILE_ENTRY(self->downloadDir),TRUE));
     gnome_config_set_string("/gnorpm/rpmfind/hostname",
 			    gtk_entry_get_text(GTK_ENTRY(self->hostname)));
     gnome_config_set_int("/gnorpm/rpmfind/expires",
@@ -782,6 +877,28 @@ static void rpm_props_box_apply(GnomePropertyBox *object, gint page_num) {
     (* parent_class->apply)(object, page_num);
 }
 
+#ifdef HAVE_RPM_3_0
+void rpm_props_box_get_flags(gint32 *interfaceFlags, gint32 *transFlags,
+			     gint32 *probFilter) {
+  gint i;
+  gchar buf[1024];
+  gint32 inter = 0, trans = 0, filter = 0;
+
+  for (i = 0; i < num_prop_flags; i++) {
+    g_snprintf(buf, 1023, "/gnorpm/Flags/%s=false", prop_flags[i].tag);
+    if (gnome_config_get_bool(buf)) {
+      inter  |= prop_flags[i].interfaceFlags;
+      trans  |= prop_flags[i].transFlags;
+      filter |= prop_flags[i].probFilter;
+    }
+  }
+  if (interfaceFlags) *interfaceFlags = inter;
+  if (transFlags)     *transFlags = trans;
+  if (probFilter)     *probFilter = filter;
+}
+
+#else /* !HAVE_RPM_3_0 */
+
 void rpm_props_box_get_flags(gint32 *interfaceFlags, gint32 *installFlags,
 			     gint32 *uninstallFlags) {
   gint i;
@@ -800,3 +917,4 @@ void rpm_props_box_get_flags(gint32 *interfaceFlags, gint32 *installFlags,
   if (installFlags)   *installFlags = inst;
   if (uninstallFlags) *uninstallFlags = uninst;
 }
+#endif /* !HAVE_RPM_3_0 */
