@@ -1,4 +1,4 @@
-/* GnoRPM - A GNOME front end for the Redhat Package Manager (RPM)
+/* GnoRPM - A GNOME front end for the Red Hat Package Manager (RPM)
  * Copyright (C) 1998-1999  James Henstridge
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,6 +24,7 @@
 #include <time.h>
 #include <fcntl.h>
 #include "pixmaps.h"
+#include "rpmcompat.h"
 
 enum {
   VERIFY,
@@ -31,6 +32,7 @@ enum {
   INSTALL,
   UPGRADE,
   CHECKSIG,
+  CLOSE,
   LAST_SIGNAL
 };
 
@@ -49,6 +51,7 @@ static void rpm_query_call_uninstall(RpmQuery *query);
 static void rpm_query_call_install(RpmQuery *query);
 static void rpm_query_call_upgrade(RpmQuery *query);
 static void rpm_query_call_checksig(RpmQuery *query);
+static void rpm_query_call_close(RpmQuery *query);
 
 static void rpm_query_set_from_header(RpmQuery *query, Header head);
 
@@ -108,6 +111,13 @@ static void rpm_query_class_init(RpmQueryClass *klass) {
 		   GTK_SIGNAL_OFFSET(RpmQueryClass, checksig),
 		   gtk_signal_default_marshaller,
 		   GTK_TYPE_NONE, 0);
+  query_signals[CLOSE] =
+    gtk_signal_new("close",
+		   GTK_RUN_FIRST,
+		   object_class->type,
+		   GTK_SIGNAL_OFFSET(RpmQueryClass, close),
+		   gtk_signal_default_marshaller,
+		   GTK_TYPE_NONE, 0);
   gtk_object_class_add_signals(object_class, query_signals, LAST_SIGNAL);
   parent_class = gtk_type_class(gtk_table_get_type());
   object_class->destroy = rpm_query_destroy;
@@ -120,6 +130,7 @@ static void rpm_query_init(RpmQuery *self) {
   GtkWidget *w, *box, *swin;
 
   table = GTK_TABLE(self);
+
   self->pkgname = gtk_label_new("<pkgname>");
   gtk_widget_set_name(self->pkgname, "PackageName");
   gtk_table_attach(table, self->pkgname, 0,4, 0,1, GTK_FILL|GTK_EXPAND,
@@ -181,6 +192,7 @@ static void rpm_query_init(RpmQuery *self) {
   gtk_widget_show(w);
   self->dist = gtk_label_new("<dist>");
   gtk_widget_set_name(self->dist, "Information");
+  gtk_label_set_line_wrap(GTK_LABEL(self->dist), TRUE);
   gtk_misc_set_alignment(GTK_MISC(self->dist), 0.0, 0.5);
   gtk_table_attach(table, self->dist, 1,2, 3,4, GTK_FILL|GTK_EXPAND,
 		   GTK_FILL, 0,0);
@@ -193,6 +205,7 @@ static void rpm_query_init(RpmQuery *self) {
   gtk_widget_show(w);
   self->vend = gtk_label_new("<vend>");
   gtk_widget_set_name(self->vend, "Information");
+  gtk_label_set_line_wrap(GTK_LABEL(self->vend), TRUE);
   gtk_misc_set_alignment(GTK_MISC(self->vend), 0.0, 0.5);
   gtk_table_attach(table, self->vend, 3,4, 3,4, GTK_FILL|GTK_EXPAND,
 		   GTK_FILL, 0,0);
@@ -205,6 +218,7 @@ static void rpm_query_init(RpmQuery *self) {
   gtk_widget_show(w);
   self->group = gtk_label_new("<group>");
   gtk_widget_set_name(self->group, "Information");
+  gtk_label_set_line_wrap(GTK_LABEL(self->group), TRUE);
   gtk_misc_set_alignment(GTK_MISC(self->group), 0.0, 0.5);
   gtk_table_attach(table, self->group, 1,2, 4,5, GTK_FILL|GTK_EXPAND,
 		   GTK_FILL, 0,0);
@@ -213,12 +227,13 @@ static void rpm_query_init(RpmQuery *self) {
   w = gtk_label_new(_("Packager:"));
   gtk_widget_set_name(w, "Heading");
   gtk_misc_set_alignment(GTK_MISC(w), 1.0, 0.5);
-  gtk_table_attach(table, w, 2,3, 4,5, GTK_FILL, GTK_FILL, 0,0);
+  gtk_table_attach(table, w, 2,3, 4,5, GTK_SHRINK, GTK_FILL, 0,0);
   gtk_widget_show(w);
 
   self->packager = gnome_href_new("<packager>", NULL);
-  gtk_table_attach(table, self->packager, 3,4, 4,5, GTK_FILL|GTK_EXPAND,
+  gtk_table_attach(table, self->packager, 3,4, 4,5, GTK_SHRINK|GTK_EXPAND,
 		   GTK_FILL, 0,0);
+  gtk_label_set_line_wrap(GTK_LABEL(GNOME_HREF(self->packager)->label), TRUE);
   gtk_widget_show(self->packager);
   /* tie visibility of label to visibility of url */
   gtk_signal_connect_object(GTK_OBJECT(self->packager), "show",
@@ -245,7 +260,7 @@ static void rpm_query_init(RpmQuery *self) {
   gtk_table_attach(table, box, 0,4, 6,7, GTK_FILL|GTK_EXPAND,
 		   GTK_FILL, 0,0);
   gtk_widget_show(box);
-  
+
   self->description = gtk_text_new(FALSE, FALSE);
   gtk_widget_set_name(self->description, "Description");
   gtk_widget_set_usize(self->description, -1, 65);
@@ -270,9 +285,10 @@ static void rpm_query_init(RpmQuery *self) {
   gtk_widget_set_name(self->files, "FileList");
   gtk_widget_set_usize(self->files, -1, 75);
   gtk_clist_column_titles_passive(GTK_CLIST(self->files));
-  gtk_clist_set_column_width(GTK_CLIST(self->files), 0, 10);
-  gtk_clist_set_column_width(GTK_CLIST(self->files), 1, 10);
-  gtk_clist_set_column_width(GTK_CLIST(self->files), 2, 10);
+  gtk_clist_set_column_min_width(GTK_CLIST(self->files), 0, 10);
+  gtk_clist_set_column_min_width(GTK_CLIST(self->files), 1, 10);
+  gtk_clist_set_column_min_width(GTK_CLIST(self->files), 2, 10);
+  gtk_clist_set_column_auto_resize(GTK_CLIST(self->files), 3, TRUE);
   gtk_container_add(GTK_CONTAINER(w), self->files);
   gtk_widget_show(self->files);
 
@@ -289,6 +305,7 @@ static void rpm_query_init(RpmQuery *self) {
   gtk_table_set_row_spacing(table, 3, 3);
   gtk_table_set_row_spacing(table, 4, 3);
   gtk_table_set_row_spacing(table, 5, 3);
+  gtk_table_set_row_spacing(table, 7, 5);
 
   self->source = RPM_QUERY_DB;
   self->hdl = NULL;
@@ -317,6 +334,7 @@ GtkWidget *rpm_query_new_from_index(DBHandle *hdl, guint index) {
   }
 
   rpm_query_add_opp_buttons(self);
+  rpm_query_add_close_button(self);
   return GTK_WIDGET(self);
 }
 
@@ -331,6 +349,7 @@ GtkWidget *rpm_query_new_from_file(DBHandle *hdl, gchar *file) {
 
   rpm_query_add_install_buttons(self);
   rpm_query_add_checksig_button(self);
+  rpm_query_add_close_button(self);
   return GTK_WIDGET(self);
 }
 
@@ -345,6 +364,7 @@ GtkWidget *rpm_query_new_from_rdf(DBHandle *hdl, rpmData *data) {
   }
 
   rpm_query_add_install_buttons(self);
+  rpm_query_add_close_button(self);
   return GTK_WIDGET(self);
 }
 #endif
@@ -381,6 +401,18 @@ void rpm_query_add_checksig_button(RpmQuery *self) {
   gtk_widget_show(w);
 }
 
+void rpm_query_add_close_button(RpmQuery *self) {
+  GtkWidget *w;
+
+  w = gnome_pixmap_button(gnome_stock_pixmap_widget(NULL,
+				GNOME_STOCK_PIXMAP_CLOSE), _("Close"));
+  gtk_signal_connect_object(GTK_OBJECT(w), "clicked",
+			    (GtkSignalFunc) rpm_query_call_close,
+			    GTK_OBJECT(self));
+  gtk_container_add(GTK_CONTAINER(self->action_area), w);
+  gtk_widget_show(w);
+}
+
 void rpm_query_add_opp_buttons(RpmQuery *self) {
   GtkWidget *w;
 
@@ -451,14 +483,15 @@ gint rpm_query_set_from_db(RpmQuery *self, DBHandle *hdl, guint index) {
 }
 
 gint rpm_query_set_from_file(RpmQuery *self, DBHandle *hdl, gchar *file) {
-  int fd;
+  FD_t fd;
   Header h = NULL;
 
-  fd = open(file, O_RDONLY);
-  if (fd < 0)
+  fd = fdOpen(file, O_RDONLY, 0);
+  if (fdFileno(fd) < 0)
     return 1;
   if (rpmReadPackageHeader(fd, &h, NULL, NULL, NULL) || !h) {
     if (!h) headerFree(h);
+    fdClose(fd);
     return 1;
   }
   rpm_query_set_from_header(self, h);
@@ -474,6 +507,7 @@ gint rpm_query_set_from_file(RpmQuery *self, DBHandle *hdl, gchar *file) {
   self->source = RPM_QUERY_FILE;
   self->fname = g_strdup(file);
   self->hdl = hdl;
+  fdClose(fd);
   return 0;
 }
 
@@ -766,4 +800,6 @@ static void rpm_query_call_upgrade(RpmQuery *query) {
 static void rpm_query_call_checksig(RpmQuery *query) {
   gtk_signal_emit(GTK_OBJECT(query), query_signals[CHECKSIG]);
 }
-
+static void rpm_query_call_close(RpmQuery *query) {
+  gtk_signal_emit(GTK_OBJECT(query), query_signals[CLOSE]);
+}
