@@ -1,4 +1,4 @@
-/* GnoRPM - A GNOME front end for the Redhat Package Manager (RPM)
+/* GnoRPM - A GNOME front end for the Red Hat Package Manager (RPM)
  * Copyright (C) 1998-1999  James Henstridge
  *
  * This program is free software; you can redistribute it and/or modify
@@ -52,7 +52,7 @@ static void rpm_web_find_class_init(RpmWebFindClass *klass) {
   
 }
 
-static void rpm_web_find_do_search(RpmWebFind *self);
+static gint rpm_web_find_do_search(RpmWebFind *self);
 static void rpm_web_find_fill_tree(RpmWebFind *self, GList *results);
 static void rpm_web_find_tree_expand(RpmWebFind *self, GtkCTreeNode *node);
 static void rpm_web_find_tree_select_row(RpmWebFind *self, GtkCTreeNode *node,
@@ -62,9 +62,11 @@ static void rpm_web_find_install(RpmWebFind *self);
 static void rpm_web_find_upgrade(RpmWebFind *self);
 
 static gboolean rpm_web_find_init_tree(RpmWebFind *self) {
+  self->filling_tree = TRUE;
   gtk_widget_set_sensitive(self->ctree, FALSE);
   rpm_web_find_fill_tree(self, getFullPackageList());
   gtk_widget_set_sensitive(self->ctree, TRUE);
+  self->filling_tree = FALSE;
   return FALSE;
 }
 
@@ -109,6 +111,7 @@ static void rpm_web_find_init(RpmWebFind *self) {
   titles[0] = _(titles[0]);
   titles[1] = _(titles[1]);
   self->ctree = gtk_ctree_new_with_titles(2, 0, titles);
+  gtk_clist_set_selection_mode(GTK_CLIST(self->ctree), GTK_SELECTION_BROWSE);
   gtk_ctree_set_indent(GTK_CTREE(self->ctree), 10);
   gtk_clist_set_column_width(GTK_CLIST(self->ctree), 0, 150);
   gtk_clist_column_titles_passive(GTK_CLIST(self->ctree));
@@ -126,12 +129,14 @@ static void rpm_web_find_init(RpmWebFind *self) {
 
   self->info = rpm_query_new();
   rpm_query_add_install_buttons(RPM_QUERY(self->info));
+  rpm_query_add_close_button(RPM_QUERY(self->info));
   gtk_signal_connect_object(GTK_OBJECT(self->info), "install",
 			    GTK_SIGNAL_FUNC(rpm_web_find_install),
 			    GTK_OBJECT(self));
   gtk_signal_connect_object(GTK_OBJECT(self->info), "upgrade",
 			    GTK_SIGNAL_FUNC(rpm_web_find_upgrade),
 			    GTK_OBJECT(self));
+  
   gtk_container_set_border_width(GTK_CONTAINER(self->info), GNOME_PAD);
   gtk_paned_add2(GTK_PANED(paned), self->info);
   gtk_widget_show(self->info);
@@ -151,6 +156,11 @@ static void rpm_web_find_init(RpmWebFind *self) {
   self->alt = NULL;
   self->name = NULL;
 
+  self->filling_tree = FALSE;
+
+  self->vc_tag = 0;
+  self->vc_node = NULL;
+
   self->install_cb = NULL;
   self->upgrade_cb = NULL;
   self->query_cb = NULL;
@@ -177,22 +187,81 @@ void rpm_web_find_set_callbacks(RpmWebFind *self,
   self->checksig_cb = checksig_cb;
 }
 
-static void rpm_web_find_do_search(RpmWebFind *self) {
+static gint rpm_web_find_do_search(RpmWebFind *self) {
   GList *results;
-  gchar *search = gtk_entry_get_text(GTK_ENTRY(self->entry));
+  gchar *search;
 
+  if (self->filling_tree) {
+    self->filling_tree = FALSE;
+    return FALSE;
+  }
+
+  self->filling_tree = TRUE;
+  search = gtk_entry_get_text(GTK_ENTRY(self->entry));
   gtk_clist_clear(GTK_CLIST(self->ctree));
   results = aproposSearch(search);
   gtk_widget_set_sensitive(self->ctree, FALSE);
   rpm_web_find_fill_tree(self, results);
   gtk_widget_set_sensitive(self->ctree, TRUE);
   freeApropos(results);
+  self->filling_tree = FALSE;
+  return FALSE;
+}
+
+static gint rpm_web_find_vcheck(RpmWebFind *self) {
+  gchar *name;
+  dbiIndexSet matches;
+
+  if (!self->vc_node || ! GTK_CTREE_ROW(self->vc_node)) {
+    self->vc_tag = 0;
+    self->vc_node = NULL;
+    return FALSE;
+  }
+  gtk_ctree_get_node_info(GTK_CTREE(self->ctree), self->vc_node, &name,
+			  NULL, NULL, NULL, NULL, NULL, NULL, NULL);
+  if (!rpmdbFindByLabel(self->hdl->db, name, &matches)) {
+    Header best = NULL;
+    gint i;
+    gchar *ver, *rel, buf[512];
+    
+    /* deduce newest version from those installed */
+    for (i = 0; i < matches.count; i++) {
+      gint index = matches.recs[i].recOffset;
+      Header h = rpmdbGetRecord(self->hdl->db, index);
+
+      if (!best)
+	best = h;
+      else {
+	if (rpmVersionCompare(best, h) < 0) {
+	  headerFree(best);
+	  best = h;
+	} else
+	  headerFree(h);
+      }
+    }
+    dbiFreeIndexRecord(matches);
+    headerGetEntry(best, RPMTAG_VERSION, NULL, (void**)&ver, NULL);
+    headerGetEntry(best, RPMTAG_RELEASE, NULL, (void**)&rel, NULL);
+    g_snprintf(buf, 511, _("have %s-%s"), ver, rel);
+    headerFree(best);
+    gtk_ctree_node_set_text(GTK_CTREE(self->ctree), self->vc_node, 1, buf);
+  }
+  self->vc_node = GTK_CTREE_ROW(self->vc_node)->sibling;
+  if (self->vc_node)
+    return TRUE;
+  self->vc_tag = 0;
+  return FALSE;
 }
 
 static void rpm_web_find_fill_tree(RpmWebFind *self, GList *results) {
   GList *tmp;
   gchar *row[2] = { "", "" };
 
+  if (self->vc_tag)
+    gtk_idle_remove(self->vc_tag);
+  self->vc_tag = 0;
+  self->vc_node = NULL;
+
   self->packageSelected = FALSE;
   self->alt = NULL;
   self->name = NULL;
@@ -213,9 +282,20 @@ static void rpm_web_find_fill_tree(RpmWebFind *self, GList *results) {
 
     while (gtk_events_pending())
       gtk_main_iteration();
+    if (!self->filling_tree) {
+      /* a search was initiated while we were filling the tree */
+      gtk_clist_select_row(GTK_CLIST(self->ctree), 0, 0);
+      gtk_clist_thaw(GTK_CLIST(self->ctree));
+      gtk_idle_add((GtkFunction)rpm_web_find_do_search, self);
+      g_message("results display was interrupted");
+      return;
+    }
   }
   gtk_clist_select_row(GTK_CLIST(self->ctree), 0, 0);
   gtk_clist_thaw(GTK_CLIST(self->ctree));
+  self->vc_node = gtk_ctree_node_nth(GTK_CTREE(self->ctree), 0);
+  if (self->vc_node)
+    self->vc_tag = gtk_idle_add((GtkFunction)rpm_web_find_vcheck, self);
 }
 
 static void rpm_web_find_tree_expand(RpmWebFind *self, GtkCTreeNode *node) {
@@ -392,3 +472,4 @@ static void rpm_web_find_upgrade(RpmWebFind *self) {
   rpm_web_find_get_packages(self, TRUE);
 }
 
+
