@@ -1,4 +1,4 @@
-/* GnoRPM - A GNOME front end for the Redhat Package Manager (RPM)
+/* GnoRPM - A GNOME front end for the Red Hat Package Manager (RPM)
  * Copyright (C) 1998-1999  James Henstridge
  *
  * This program is free software; you can redistribute it and/or modify
@@ -44,6 +44,15 @@ struct _RpmWebFind {
   rpmPackageAlternate *alt;
   gchar *name;
 
+  /* this is used to indicate that the tree is being filled -- used to
+   * allow interruption filling the ctree with results (so we don't get
+   * a mix of two sets of results */
+  gboolean filling_tree;
+
+  /* this is for the version checker */
+  guint vc_tag;
+  GtkCTreeNode *vc_node;
+
   /* callbacks for the created install dialog ... */
   GtkSignalFunc install_cb, upgrade_cb, query_cb, checksig_cb;
 };
